use clap::Parser;
use egui::IconData;
use libpt::cli::args::{VerbosityLevel, HELP_TEMPLATE};
use libpt::log::{debug, trace};

pub const TITLE: &str = "Rollator";

/// Placeholder comment that will show in the help in the CLI
///
/// amogus
#[derive(serde::Deserialize, serde::Serialize, Debug, Clone, Parser)]
#[command(help_template = HELP_TEMPLATE, author, version)]
#[serde(default)] // if we add new fields, give them default values when deserializing old state
pub struct RollatorApp {
    #[clap(skip)]
    label: String,

    #[serde(skip)]
    #[clap(skip)]
    value: f32,

    #[serde(skip)]
    #[command(flatten)]
    pub(crate) verbosity: VerbosityLevel,

    #[serde(skip)]
    #[clap(skip)]
    show_info_window: bool,
}

impl Default for RollatorApp {
    fn default() -> Self {
        Self {
            label: TITLE.to_owned(),
            value: 2.7,
            verbosity: VerbosityLevel::INFO,
            show_info_window: false,
        }
    }
}

impl RollatorApp {
    pub fn restore_state(&mut self, cc: &eframe::CreationContext<'_>) {
        // Load previous app state (if any).
        // Note that you must enable the `persistence` feature for this to work.
        if let Some(storage) = cc.storage {
            let old: Self = eframe::get_value(storage, eframe::APP_KEY).unwrap_or_default();

            self.label = old.label;
            self.value = old.value;
        }
    }

    /// Called once before the first frame.
    #[allow(unused)] // can be used for starting with wasm
    pub fn new() -> Self {
        Default::default()
    }

    pub fn new_with_cli() -> Self {
        Self::parse()
    }

    pub fn info_diag(&mut self, ctx: &egui::Context) {
        trace!("rendering info dialogue");
        ctx.show_viewport_immediate(
            egui::ViewportId::from_hash_of("immediate_viewport"),
            egui::ViewportBuilder::default()
                .with_title("Immediate Viewport")
                .with_inner_size([500.0, 200.0]),
            |ctx, class| {
                assert!(
                    class == egui::ViewportClass::Immediate,
                    "This egui backend doesn't support multiple viewports"
                );

                egui::CentralPanel::default().show(ctx, |ui| {
                    ui.spacing_mut().item_spacing.x = 0.0;
                    ui.label(format!("{TITLE} v{}", env!("CARGO_PKG_VERSION")));
                    ui.hyperlink_to("Source Code\n", env!("CARGO_PKG_REPOSITORY"));
                    ui.label(format!("Author: {}", env!("CARGO_PKG_AUTHORS")));
                    ui.label(format!("License: {}", env!("CARGO_PKG_LICENSE")));
                    ui.label(format!(
                        "\n{TITLE} is free software. If you paid for this you were scammed.\n"
                    ));

                    ////////////////////////////////////////////////////////////////

                    bottom_label(ui);

                    ////////////////////////////////////////////////////////////////
                });
                if ctx.input(|i| i.viewport().close_requested()) {
                    // Tell parent viewport that we should not show next frame:
                    self.show_info_window = false;
                }
            },
        );
    }
}

impl eframe::App for RollatorApp {
    /// Called by the frame work to save state before shutdown.
    fn save(&mut self, storage: &mut dyn eframe::Storage) {
        eframe::set_value(storage, eframe::APP_KEY, self);
    }

    /// Called each time the UI needs repainting, which may be many times per second.
    fn update(&mut self, ctx: &egui::Context, _frame: &mut eframe::Frame) {
        // Put your widgets into a `SidePanel`, `TopBottomPanel`, `CentralPanel`, `Window` or `Area`.
        // For inspiration and more examples, go to https://emilk.github.io/egui

        egui::TopBottomPanel::top("top_panel").show(ctx, |ui| {
            // The top panel is often a good place for a menu bar:

            egui::menu::bar(ui, |ui| {
                // NOTE: no File->Quit on web pages!
                let is_web = cfg!(target_arch = "wasm32");
                if !is_web {
                    ui.menu_button("File", |ui| {
                        if ui.button("Quit").clicked() {
                            ctx.send_viewport_cmd(egui::ViewportCommand::Close);
                        }
                        if ui.button("Info").clicked() {}
                    });
                    ui.add_space(16.0);
                }
                if self.show_info_window {
                    self.info_diag(ctx);
                }

                egui::widgets::global_dark_light_mode_buttons(ui);
            });
        });

        egui::CentralPanel::default().show(ctx, |ui| {
            // The central panel the region left after adding TopPanel's and SidePanel's
            ui.heading(TITLE);

            ui.horizontal(|ui| {
                ui.label("Write something: ");
                ui.text_edit_singleline(&mut self.label);
            });

            ui.add(egui::Slider::new(&mut self.value, 0.0..=10.0).text("value"));
            if ui.button("Increment").clicked() {
                self.value += 1.0;
            }

            ui.separator();

            ui.with_layout(egui::Layout::bottom_up(egui::Align::LEFT), |ui| {
                bottom_label(ui);
                egui::warn_if_debug_build(ui);
            });
        });
    }
}

fn bottom_label(ui: &mut egui::Ui) {
    ui.horizontal(|ui| {
        ui.spacing_mut().item_spacing.x = 0.0;
        ui.hyperlink_to("Source Code ", env!("CARGO_PKG_REPOSITORY"));
        ui.label("| Powered by ");
        ui.hyperlink_to("egui", "https://github.com/emilk/egui");
        ui.label(" and ");
        ui.hyperlink_to(
            "eframe",
            "https://github.com/emilk/egui/tree/master/crates/eframe",
        );
        ui.label(".");
    });
}

#[allow(unused)] // it is used in the main function
pub(crate) fn load_icon() -> IconData {
    const ICON_RAW: &[u8; 27618] = include_bytes!("../assets/icon-512.jpg");
    let (icon_rgba, icon_width, icon_height) = {
        let image = image::load_from_memory(ICON_RAW)
            .expect("Failed to open icon path")
            .into_rgba8();
        let (width, height) = image.dimensions();
        let rgba = image.into_raw();
        (rgba, width, height)
    };

    IconData {
        rgba: icon_rgba,
        width: icon_width,
        height: icon_height,
    }
}
